@@ -77,7 +77,6 @@ static void on_alarm(void *acp, int occured) {
   async_connect *ac = acp;
   gpr_mu_lock(&ac->mu);
   /* If the alarm didn't occur, it got cancelled. */
-  gpr_log(GPR_DEBUG, "on_alarm: %p", ac->socket);
   if (ac->socket != NULL && occured) {
     grpc_winsocket_shutdown(ac->socket);
   }
@@ -96,8 +95,6 @@ static void on_connect(void *acp, int from_iocp) {
 
   gpr_mu_lock(&ac->mu);
 
-  gpr_log(GPR_DEBUG, "on_connect: %p", ac->socket);
-
   if (from_iocp) {
     DWORD transfered_bytes = 0;
     DWORD flags;
