@@ -44,14 +44,14 @@ namespace node {
 /* Wraps grpc_server as a JavaScript object. Provides a constructor
    and wrapper methods for grpc_server_create, grpc_server_request_call,
    grpc_server_add_http2_port, and grpc_server_start. */
-class Server : public ::node::ObjectWrap {
+class Server : public Nan::ObjectWrap {
  public:
   /* Initializes the Server class and exposes the constructor and
      wrapper methods to JavaScript */
-  static void Init(v8::Handle<v8::Object> exports);
+  static void Init(v8::Local<v8::Object> exports);
   /* Tests whether the given value was constructed by this class's
      JavaScript constructor */
-  static bool HasInstance(v8::Handle<v8::Value> val);
+  static bool HasInstance(v8::Local<v8::Value> val);
 
  private:
   explicit Server(grpc_server *server);
@@ -69,8 +69,8 @@ class Server : public ::node::ObjectWrap {
   static NAN_METHOD(Start);
   static NAN_METHOD(TryShutdown);
   static NAN_METHOD(ForceShutdown);
-  static NanCallback *constructor;
-  static v8::Persistent<v8::FunctionTemplate> fun_tpl;
+  static Nan::Callback *constructor;
+  static Nan::Persistent<v8::FunctionTemplate> fun_tpl;
 
   grpc_server *wrapped_server;
   grpc_completion_queue *shutdown_queue;
