@@ -67,6 +67,13 @@ abstract class AbstractCall {
     return $this->metadata;
   }
 
+  /**
+   * @return string The URI of the endpoint.
+   */
+  public function getPeer() {
+    return $this->call->getPeer();
+  }
+
   /**
    * Cancels the call
    */
@@ -85,4 +92,4 @@ abstract class AbstractCall {
     }
     return call_user_func($this->deserialize, $value);
   }
-}
\ No newline at end of file
+}
