@@ -46,6 +46,3 @@ cd ext/grpc
 phpize
 ./configure --enable-grpc=$root
 make
-
-# in some jenkins macos machine, somehow the PHP build script can't find libgrpc.dylib
-export DYLD_LIBRARY_PATH=$(pwd)/libs/$config
